"""
AIPTX Beast Mode - WAF Module
=============================

Web Application Firewall fingerprinting and bypass strategies.
"""

from __future__ import annotations

from aipt_v2.exploitation.waf.fingerprinter import (
    WAFFingerprinter,
    fingerprint_waf,
)
from aipt_v2.exploitation.waf.signatures import (
    WAF_SIGNATURES_DB,
    WAFSignatureEntry,
    get_signature,
)
from aipt_v2.exploitation.waf.bypass_strategies import (
    BypassStrategy,
    BYPASS_STRATEGIES,
    get_bypass_strategy,
    get_all_strategies,
)

__all__ = [
    "WAFFingerprinter",
    "fingerprint_waf",
    "WAF_SIGNATURES_DB",
    "WAFSignatureEntry",
    "get_signature",
    "BypassStrategy",
    "BYPASS_STRATEGIES",
    "get_bypass_strategy",
    "get_all_strategies",
]
